@@ -1,5 +1,8 @@
 # -*- coding: utf-8 -*-
+from datetime import datetime, timedelta
+
 from django.http import HttpResponseNotAllowed
+from django.utils import timezone
 from django.utils.deprecation import MiddlewareMixin
 
 from .helpers import User, check_allow_for_uri, check_allow_for_user
@@ -12,6 +15,22 @@ class ImpersonateMiddleware(MiddlewareMixin):
         request.impersonator = None
 
         if request.user.is_authenticated and '_impersonate' in request.session:
+            if settings.MAX_DURATION:
+                if '_impersonate_start' not in request.session:
+                    return
+
+                start_time = datetime.fromtimestamp(
+                    request.session['_impersonate_start'], timezone.utc
+                )
+
+                if datetime.now(timezone.utc) - start_time > timedelta(
+                    seconds=settings.MAX_DURATION
+                ):
+                    del request.session['_impersonate']
+                    del request.session['_impersonate_start']
+
+                    return
+
             new_user_id = request.session['_impersonate']
             if isinstance(new_user_id, User):
                 # Edge case for issue 15
